import os
import time
import pickle
import datetime
from threading import Thread
from dataEngine import Logger, AppConfig

logger = Logger.get()
ConferencesList = {}


class Recording:
    __slots__ = ("start_time", "last_activity_time", "stop_time", "speakers")

    def __init__(self):
        self.start_time = datetime.datetime.now()
        self.last_activity_time = None
        self.stop_time = None
        self.speakers = {}

    def start(self):
        # here it's needed to execute "confbridge record start <conf_id>" via AMI interface
        logger.notice("confdbg: recording has been started")

    def stop(self):
        # here it's needed to execute "confbridge record stop <conf_id>" via AMI interface
        self.stop_time = datetime.datetime.now()
        logger.notice("confdbg: recording has been stopped")

    def log_channel_action(self, channel, action):
        if action == "starttalking":
            if channel not in self.speakers:
                self.speakers[channel] = Speaker(channel)
            else:
                self.speakers[channel].start_talking()
        elif action == "stoptalking":
            if channel not in self.speakers:
                logger.error("confdbg: channel '%s' stopped talking, but speaker object is absent!" % channel)
                return
            self.speakers[channel].stop_talking()


class Speaker:
    __slots__ = ("channel", "activity_details")

    def __init__(self, channel):
        # self.exten = None
        # self.name = None
        self.channel = channel
        self.activity_details = [{"start": datetime.datetime.now()}]

    def start_talking(self):
        if not self.activity_details[-1]["stop"]:
            logger.error("confdbg: channel %s started talking again, but the stop time of previous object is absent"
                         % self.channel)
        self.activity_details.append({"start": datetime.datetime.now()})

    def stop_talking(self):
        if "stop" in self.activity_details[-1]:
            logger.error("confdbg: channel %s stopped talking, but the last object already has stop_time"
                         % self.channel)
        self.activity_details[-1]["stop"] = datetime.datetime.now()


class Conference:
    __slots__ = ("conf_id", "active_channels", "record")

    def __init__(self, conf_id):
        self.conf_id = conf_id
        self.active_channels = []
        self.record = None

    def __str__(self):
        return "confdbg: conference object (conference id %s). Status: %s (%s). Recorded: %s." % \
               (self.conf_id, "active" if self.is_active() else "inactive", len(self.active_channels),
                "yes" if self.record else "no")

    def is_active(self):
        return True if self.active_channels else False

    def is_recorded(self):
        return self.record is not None

    def store_data(self):
        tmp_dir = "/var/spool/callweaver/monitor/conf_details/"
        if not os.path.exists(tmp_dir):
            os.mkdir(tmp_dir)
        filename = "conf_%s_%s" % (self.conf_id, datetime.datetime.now().timestamp())
        with open(os.path.join(tmp_dir, filename), 'wb') as f:
            pickle.dump(self.record, f)
        logger.notice("confdbg: recording object has been dumped")

    def add_speaker(self, channel):
        logger.notice("confdbg: channel %s has started talking" % channel)
        if channel not in self.active_channels:
            self.active_channels.append(channel)
        else:
            logger.error("confdbg: channel '%s' has been already added to active channels list!" % channel)

        if not self.record:
            self.record = Recording()
            self.record.start()

        self.record.log_channel_action(channel, "starttalking")

    def remove_speaker(self, channel):
        logger.notice("confdbg: channel %s has stopped talking" % channel)
        if channel not in self.active_channels:
            logger.error("confdbg: channel '%s' is absent in active channels list!" % channel)
            return

        self.active_channels.remove(channel)
        if not self.is_active():
            self.record.last_activity_time = datetime.datetime.now()

        if not self.record:
            logger.error("confdbg: channel '%s' stopped talking, but the recording hasn't been started yet!" % channel)
            return

        self.record.log_channel_action(channel, "stoptalking")


def get(conf_id):
    if conf_id not in ConferencesList:
        ConferencesList[conf_id] = Conference(conf_id)
    return ConferencesList[conf_id]


def stop_inactive_recordings():
    delay = AppConfig.get("conf_seconds_before_stop_recording")
    for conf_id in ConferencesList:
        if ConferencesList[conf_id].is_recorded() and not ConferencesList[conf_id].is_active() and \
                datetime.datetime.now() - ConferencesList[conf_id].record.last_activity_time > \
                datetime.timedelta(seconds=delay):
            ConferencesList[conf_id].record.stop()
            ConferencesList[conf_id].store_data()
            ConferencesList[conf_id].record = None


def recordings_timer():
    while True:
        time.sleep(5)
        stop_inactive_recordings()


def main():
    stop_recording = Thread(target=recordings_timer)
    stop_recording.daemon = True
    stop_recording.start()


main()
