
import httplib2
import json
import time
import os
from threading import Thread
from dataEngine import Logger, Common, Driver

logger = Logger.get()
USER_AGENT = os.environ.get("USER_AGENT", "Wildix W-AIR-SP/HS,Wildix W-AIR-SBR2/HS")
CONF_EXTEN = os.environ.get("CONF_EXTEN", "*Wairhotline*")
UsersList = {}
periodic_checker = None

class User(dict):
    def __init__(self, user_id):
        self.user_id = user_id
        self.channels = []
        self.foreign_channels = []
        self.contact = None
        self.charger_state = None
        self.try_after = int(time.time()) + 2
        self.call_attempts = 0
        self.hold = False

    def __str__(self):
        return "user id: {}.\nchannels: {}.\nforeign_channels: {}.\ncontact: {}.\ncharger_state: {}.\nhold: {}.\ntry_after: {}.\ncall_attempts: {}".\
            format(self.user_id, self.channels, self.foreign_channels, self.contact, self.charger_state, self.hold, self.try_after, self.call_attempts)

    def get(self, param):
        return getattr(self, param, None)

    def set(self, param, value):
        setattr(self, param, value)

    def add_channel(self, chan_name):
        if chan_name not in self.channels:
            self.channels.append(chan_name)

    def remove_channel(self, chan_name):
        self.channels.remove(chan_name)

    def add_foreign_channel(self, chan_name):
        if chan_name not in self.foreign_channels:
            self.foreign_channels.append(chan_name)

    def remove_foreign_channel(self, chan_name):
        if chan_name in self.foreign_channels:
            self.foreign_channels.remove(chan_name)


def _get_user(user_id):
    if user_id not in UsersList:
        UsersList[user_id] = User(user_id)
    return UsersList[user_id]


def _is_xhoppers_device(useragent):
    if not useragent:
        return False
    for pattern in USER_AGENT.split(","):
        logger.debug("is_xhoppers_device. Useragent [ {} ] match pattern [ {} ]".format(useragent, pattern))
        if pattern in useragent:
            logger.debug("is_xhoppers_device: True.")
            return True


def _http_client(url):
    if not url:
        return
    h = httplib2.Http(cache=None, timeout=0.5)
    try:
        resp, content = h.request(url, "GET")
    except Exception as e:
        logger.error(e)
        return []
    if not content:
        logger.notice("Can't get result for request to {}".format(url))
        return []
    try:
        response = json.loads(content.decode('utf-8'))
    except Exception as e:
        logger.error(e)
        return []
    if response.get("type") == "error":
        logger.notice("Can't get data: {}".format(response.get("reason")))
        return []
    return response.get("result")


def _put_to_conference(user):
    if int(time.time()) < user.get("try_after"):
        return
    if user.get("contact") is None:
        return
    if user.get("charger_state") == "on-charger":
        return
    if len(user.get("foreign_channels")) >= 1:
        return

    if len(user.get("channels")) < 1:
        logger.debug("Put user {}, to the x-hoppers conference".format(user.get("user_id")))
        Driver.get_conn().send_action('SendRefer', Dest=user.get("user_id"), To=CONF_EXTEN, UaID=user.get("contact"))
        user.set("call_attempts", user.get("call_attempts") + 1)
        # increase timeout between attempts
        user.set("try_after", int(time.time()) + 2**user.get("call_attempts"))


def _hungup_all_calls(user):
    for chan in user.get("channels"):
        logger.debug("Send hangup for channel {}".format(chan))
        Driver.get_conn().send_action("Hangup", Channel=chan)


def _unhold(user):
    if len(user.get("foreign_channels")) != 0:
        return
    if len(user.get("channels")) != 1:
        return
    chan_name = user.get("channels")[0]
    logger.debug("Unhold x-hoppers user {} channel {}".format(user.get("user_id"), chan_name))
    Driver.get_conn().send_action("SIPNotifyBroadsoft", Channel=chan_name, Event="talk")
    print_user(user)


def _check_user_id(user_id):
    if not user_id or "unknown" in user_id:
        return False
    return True


def process_event_reg(event_type, user_id, useragent, contact):
    if not event_type or not _check_user_id(user_id) or not useragent\
            or not contact or not _is_xhoppers_device(useragent):
        return
    user = _get_user(user_id)
    if event_type == "add":
        logger.debug("User {} registered, contact {}".format(user_id, contact))
        user.set("contact", contact)
    elif event_type == "remove":
        logger.debug("User {} un-registered".format(user_id))
        user.set("contact", None)
        # ToDo need to decide, if we want to auto set "on-charger" after losing registration
        #user.set("charger_state", "on-charger")
        _hungup_all_calls(user)
    print_user(user)


def add_chan(user_id, chan_name, exten, is_on_hold="false"):
    if not _check_user_id(user_id) or not chan_name:
        return
    user = None
    if exten == CONF_EXTEN:
        logger.debug("x-hoppers channel {} has been added".format(chan_name))
        user = _get_user(user_id)
        user.add_channel(chan_name)
    elif exten in UsersList or user_id in UsersList:
        logger.debug("x-hoppers foreign channel {} has been added".format(chan_name))
        user = _get_user(exten if exten in UsersList else user_id)
        user.add_foreign_channel(chan_name)
        return
    else:
        return
    user.set("charger_state", "off-charger")
    user.set("call_attempts", 0)
    user.set("hold", True if is_on_hold == "true" else False)
    user.set("try_after", int(time.time()) + 2 ** (user.get("call_attempts") + 1))

    if len(user.get("channels")) > 1:
        chan_name = user.get("channels")[-1:][0]
        logger.debug("Detected more than one channels in the x-hoppers conference for the user {}."
                     "Sending hangup for the channel {}".format(user_id, chan_name))
        Driver.get_conn().send_action("Hangup", Channel=chan_name)
    print_user(user)


def remove_chan(caller_id, callee_id, chan_name):
    user_id = None
    if caller_id and caller_id in UsersList:
        user_id = caller_id
    elif callee_id and callee_id in UsersList:
        user_id = callee_id
    else:
        return

    user = _get_user(user_id)
    if chan_name in user.get("channels"):
        logger.debug("X-hoppers channel {} has been removed".format(chan_name))
        user.remove_channel(chan_name)
        user.set("try_after", int(time.time()) + 2**(user.get("call_attempts") + 1))
    if chan_name in user.get("foreign_channels"):
        logger.debug("X-hoppers foreign channel {} has been removed".format(chan_name))
        user.remove_foreign_channel(chan_name)
    _unhold(user)
    print_user(user)


def set_charger_state(user_id, useragent, charger_state):
    if not _check_user_id(user_id) or not useragent or not charger_state\
            or not _is_xhoppers_device(useragent):
        return
    logger.debug("X-hoppers user {}, state is {}".format(user_id, charger_state))
    user = _get_user(user_id)
    user.set("charger_state", charger_state)
    if charger_state == "on-charger":
        _hungup_all_calls(user)
    print_user(user)


def toggle_hold_state(user_id, channel, state):
    if not user_id or not channel or not state:
        return
    if user_id not in UsersList:
        return
    user = _get_user(user_id)
    if channel not in user.get("channels"):
        return
    user.set("hold", True if state == "hold" else False)


def periodic_check():
    while Common.check_license("xhoppers"):
        time.sleep(5)
        for _, user in UsersList.items():
            _put_to_conference(user)
            if user.get("hold"):
                _unhold(user)


def init():
    global periodic_checker, UsersList
    if not Common.check_license("xhoppers"):
        if periodic_checker and periodic_checker.is_alive():
            logger.debug("Init. XHoppers license is not active, stopping periodic checker thread")
            periodic_checker.join()
            periodic_checker = None
        for _, user in UsersList.items():
            user.set("contact", None)
            _hungup_all_calls(user)
        if len(UsersList) > 0:
            UsersList = {}
        return
    logger.debug("Init. Get channels list")
    Driver.get_conn().send_action("CoreShowChannels")
    logger.debug("Init. Get charger state")
    for item in _http_client("http://localhost:5060/v1/wair_charger_state/"):
        user_id = item["user"]
        if _check_user_id(user_id):
            _get_user(user_id).set("charger_state", item["state"])
    logger.debug("Init. Get registrations")
    for aor in _http_client("http://localhost:5060/v1/wair_devices/"):
        if _is_xhoppers_device("Wildix {}/{}".format(aor.get("base_model"), aor.get("model"))):
            user_id = aor.get("extension")
            if _check_user_id(user_id):
                _get_user(user_id).set("contact", aor.get("contact"))
    print_user("all")
    if periodic_checker is None:
        logger.debug("Init. Starting Xhoppers periodic checker thread")
        periodic_checker = Thread(target=periodic_check, daemon=True, name="XHoppers periodic checker")
        periodic_checker.start()


def print_user(param):
    if param == "all":
        for _, user in UsersList.items():
            logger.debug(user)
    else:
        logger.debug(param)
